/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.common;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRField;

/**
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public final class MetaAndTableJRDataSource implements JRDataSource {

    private final List<String[]> data = new ArrayList<>();

    private final Map<String, Object> metaData = new HashMap<>();

    private int index = -1;

    public void addData(final String[] row) {
        this.data.add(row);
    }

    public void addMetaData(final String key, final Object value) {
        this.metaData.put(key, value);
    }

    @Override
    public boolean next() throws JRException {
        this.index++;

        return this.index < this.data.size();
    }

    @Override
    public Object getFieldValue(final JRField field) throws JRException {
        final String fieldName = field.getName();

        if (fieldName.startsWith("meta:"))
            return this.metaData.get(fieldName.substring("meta:".length()));

        if (fieldName.startsWith("data:")) {
            final int column = Integer.valueOf(fieldName.substring("data:".length()));
            if (column >= this.data.get(this.index).length)
                return null;
            return this.data.get(this.index)[column];
        }
        return null;
    }

    public JRDataSource withPagePageOffsets(final Integer startpageoffset, final Integer totalpages) {

        final MetaAndTableJRDataSource newSource = new MetaAndTableJRDataSource();
        newSource.data.addAll(this.data);
        newSource.metaData.putAll(this.metaData);

        newSource.addMetaData("startpageoffset", startpageoffset);
        newSource.addMetaData("totalpages", totalpages);

        return newSource;
    }
}